﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Emgu.CV.Structure;
using Emgu.CV;
using Emgu.CV.UI;
using System.Drawing;
using System.Runtime.InteropServices;
using Emgu.CV.Util;


namespace TP
{
    /* -----------------------------------------------------------------------------------------------------
     * Classe contenant les donnees d'une image et effectuant sa gestion. 
     *  -----------------------------------------------------------------------------------------------------*/
     class VecteurCaracteristiqueImage
    {
        private Image<Bgr, Byte> imageNir;
        private Image<Bgr, Byte> imageUv;
        typeGraine typeG = new typeGraine();

        /* ----------------------------------------------------------------------------------------------------------
         *  Initialisation de la classe 
         *  ---------------------------------------------------------------------------------------------------------*/

         public VecteurCaracteristiqueImage(Image<Bgr, Byte> image)
         {       
             imageNir = image;
         }

         public VecteurCaracteristiqueImage(Image<Bgr, Byte> image, typeGraine t)
         {
             imageNir = image;
             typeG = t;
         }

         public VecteurCaracteristiqueImage(Image<Bgr, byte> imageNir, Image<Bgr, byte> imageUv, typeGraine t)
         {
             this.imageNir = imageNir;
             this.imageUv = imageUv;
             typeG = t;
         }


         /* ----------------------------------------------------------------------------------------------------------
          *  Fonction detectant la moyenne de la casette complete
          *  ---------------------------------------------------------------------------------------------------------*/
         public double[] DetecterMoyenne()
         {
             Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();

             double moyenne, moyenne_debut, moyenne_fin;
             double somme = 0, somme_debut = 0, somme_fin = 0;

             for (int i = 0; i < imageRGBgray.Height; i++)
                 for (int j = 0; j < imageRGBgray.Width; j++)
                 {
                     if (i < 2) 
                         somme_debut += imageRGBgray.Data[i, j, 0]; /*2 premières rangées*/
                     
                     somme += imageRGBgray.Data[i, j, 0];
                     
                     if (i >= imageRGBgray.Height - 2) 
                         somme_fin += imageRGBgray.Data[i, j, 0]; /*deux dernières rangés*/
                 }
             moyenne_debut = somme_debut /(imageRGBgray.Width * 2);         /*moyenne des pixels au début (2 premières rangées)*/
             moyenne = somme / (imageRGBgray.Height * imageRGBgray.Width);  /*moyenne des pixels au milieu*/
             moyenne_fin = somme_fin / (imageRGBgray.Width * 2);            /*moyenne des pixels à la fin*/

             return new double[3] { moyenne_debut, moyenne, moyenne_fin };
         }

         /* ----------------------------------------------------------------------------------------------------------
         *  Fonction calculant la variance de la casette complete
         *  ---------------------------------------------------------------------------------------------------------*/
         public double DetecterStd(double moyenne)
         {
             Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();

             double var;
             double somme = 0;

             for (int i = 0; i < imageRGBgray.Height; i++)
                 for (int j = 0; j < imageRGBgray.Width; j++)
                 {
                     somme = somme + ((imageRGBgray.Data[i, j, 0]-moyenne) * (imageRGBgray.Data[i, j, 0]-moyenne));
                     
                 }
             var = somme / (imageRGBgray.Height * imageRGBgray.Width);

             return Math.Sqrt(var);
         }

         /* ----------------------------------------------------------------------------------------------------------
   *  Fonction calculant la variance des pixels de spheigne considerant que le contour est propre donc < moyenne
   *  ---------------------------------------------------------------------------------------------------------*/
         public double DetecterStdSpheigne(double moyenne)
         {
             Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();

             double var;
             double somme = 0;
             int nb = 0;

             for (int i = 0; i < imageRGBgray.Height; i++)
                 for (int j = 0; j < imageRGBgray.Width; j++)
                 {
                     if (imageRGBgray.Data[i, j, 0] > moyenne)
                     {
                         nb++;
                         somme = somme + ((imageRGBgray.Data[i, j, 0] - moyenne) * (imageRGBgray.Data[i, j, 0] - moyenne));
                     }

                 }
             var = somme / (double)nb;

             return Math.Sqrt(var);
         }
		 
	/* ----------------------------------------------------------------------------------------------------------
   *  Fonction calculant la variance des pixels de spheigne considerant une bordure de 10 en x et y du contour de platic enleve
   *  ---------------------------------------------------------------------------------------------------------*/
         public double EstimerStdSpheigne(double moyenne)
         {
             Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();

             double var;
             double somme = 0;
             int nb = 0;

             for (int i = 10; i < imageRGBgray.Height-10; i++)
                 for (int j = 10; j < imageRGBgray.Width-10; j++)
                 {
                         nb++;
                         somme = somme + ((imageRGBgray.Data[i, j, 0] - moyenne) * (imageRGBgray.Data[i, j, 0] - moyenne));
                 }
             }
             var = somme / (double)nb;

             return Math.Sqrt(var);
         }

         /* ----------------------------------------------------------------------------------------------------------
         *  Fonction calculant la moyenne des pixels de spheigne considerant que le contour est propre donc < moyenne
         *  ---------------------------------------------------------------------------------------------------------*/
         public double DetecterMoyenneSpheigne(double moyenne)
         {
             Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();

             double moy;
             double somme = 0;
             int nb = 0;

             for (int i = 0; i < imageRGBgray.Height; i++)
                 for (int j = 0; j < imageRGBgray.Width; j++)
                 {
                     if (imageRGBgray.Data[i, j, 0] > moyenne)
                     {
                         somme = somme + imageRGBgray.Data[i, j, 0];
                         nb++;
                     }

                 }
             moy = somme / (double)nb;

             return moy;
         }
		 
		 /* ----------------------------------------------------------------------------------------------------------
         *  Fonction calculant la moyenne des pixels de spheigne, bordure de 10 en x et y du contour de platic enleve 
         *  ---------------------------------------------------------------------------------------------------------*/
         public double EstimerMoyenneSpheigne()
         {
             Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();

             double moy;
             double somme = 0;
             int nb = 0;

             for (int i = 10; i < imageRGBgray.Height-10; i++)
                 for (int j = 10; j < imageRGBgray.Width-10; j++)
                 {
                         somme = somme + imageRGBgray.Data[i, j, 0];
                         nb++;
                 }
             }
             moy = somme / (double)nb;

             return moy;
         }



         /* ----------------------------------------------------------------------------------------------------------
          *  Fonction detectant la graine mai 2015
          *  ---------------------------------------------------------------------------------------------------------*/
         public List<Graine> DetecterGraine(double moyenne,decimal hminG, decimal hmaxG, decimal lminG, decimal lmaxG)
         { 
            List<Graine> ListeGraine = new List<Graine>();
            double var = DetecterStd(moyenne);
            Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();


            imageRGBgray._ThresholdBinary(new Gray(moyenne + 50), new Gray(256)); // On rajoute 50

             
             using (MemStorage storage = new MemStorage()) //allocate storage for contour approximation
             // on detecte les contour selon une grandeur de graine x
             for (Contour<Point> contours = imageRGBgray.FindContours(); contours != null; contours = contours.HNext)
             {
                 if (contours.Total >= 6 && contours.Area > ((float)lminG * (float)hminG * Math.PI) && contours.BoundingRectangle.Width > (float)lminG && contours.BoundingRectangle.Width < (float)lmaxG &&
                          contours.BoundingRectangle.Height > (float)hminG && contours.BoundingRectangle.Height < (float)hmaxG)
                 {

                     bool ok = true;
                     if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height -10 )
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height - 10 )
                         ok = false;


 
                     // si la graine se trouve a 10 pixel des coins du contours
                     if (ok )
                    {
                        Graine gr = new Graine(contours, 1);
                      //  if (gr.Localisation.X > 1  &&gr.Localisation.Y > 1 && imageRGBgray.Width - (gr.Localisation.X + contours.BoundingRectangle.Width) > 1
                       //       && imageRGBgray.Height - (gr.Localisation.Y + contours.BoundingRectangle.Height) > 1)
                      //  {
                         Seq<Emgu.CV.Structure.MCvConvexityDefect> te = contours.GetConvexityDefacts(storage, Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);
                            MCvConvexityDefect[] defacts = te.ToArray();
                            List<double> angle = new List<double>();
                            foreach (MCvConvexityDefect d in defacts)
                            {
                                double fA = Math.Sqrt(Math.Pow(d.EndPoint.X - d.DepthPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.DepthPoint.Y, 2)); //distance entre b et c
                                double fB = Math.Sqrt(Math.Pow(d.EndPoint.X - d.StartPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.StartPoint.Y, 2));  //distance entre a et c
                                double fC = Math.Sqrt(Math.Pow(d.DepthPoint.X - d.StartPoint.X, 2) + Math.Pow(d.DepthPoint.Y - d.StartPoint.Y, 2));
                                double fAngle = (Math.Pow(fA, 2) + Math.Pow(fC, 2) - Math.Pow(fB, 2)) / (2 * fA * fC); //(a2 + c2 - b2) / (2*a*c)
                                if (Math.Abs(fAngle) < 0.60)
                                    angle.Add(fAngle);
                            }
                            //  System.Console.WriteLine(angle.Count);
                            gr.Nb_graine = angle.Count > 0 ? gr.Nb_graine + (int)Math.Ceiling(angle.Count / (double)2) : gr.Nb_graine;
                            ListeGraine.Add(gr);
                        }
                 //  }
                 }
             }
             return ListeGraine;
         }

         public double TrouverSeuilOptimalCassette(Image<Gray, Byte> im1, double moyenne)
         {

             try
             {
                 int[] hist = new int[256];

                 for (int k = 0; k < 256; k++)
                 {
                     hist[k] = 0;
                 }

                 for (int i = im1.ROI.Y; i < im1.ROI.Height; i++)
                 {
                     for (int j = im1.ROI.X; j < im1.ROI.Width; j++)
                     {
                         int val = im1.Data[i, j, 0];
                         hist[val]++;
                     }
                 }

                 double sumT = 0.0;

                 for (int t = 0; t < 256; t++)
                     sumT += (double)(t) * (double)(hist[t]);

                 double sumB = 0.0;
                 int wB = 0;
                 int wF = 0;

                 double varmax = 0.0;
                 int tOptimal = 0;
				 
				 for (int t = 0; t < (int)moyenne; t++)
                 {
                     wB += hist[t];
				 }

                 for (int t = (int)moyenne; t < 256; t++)
                 {
                     wB += hist[t];
                     if (wB == 0)
                         continue;

                     wF = im1.Width * im1.Height - wB;
                     if (wF == 0)
                         break;

                     sumB += (double)(t) * (double)(hist[t]);

                     double mB = sumB / (double)(wB);
                     double mF = (sumT - sumB) / (double)(wF);

                     double varbetween = (double)(wB) * (double)(wF) * (mB - mF) * (mB - mF);

                     if (varbetween > varmax)
                     {
                         varmax = varbetween;
                         tOptimal = t;
                     }
                 }


                 return tOptimal;

             }
             catch
             {
                 System.Console.WriteLine("Incapable de trouver le seuil optimal de Otsu");
                 return 0;
             }
         }


         public double DetecterUniformite(Image<Gray, Byte> im1, double moyenne)
         {

             try
             {
                 double[] hist = new double[256];

                 for (int k = 0; k < 256; k++)
                 {
                     hist[k] = 0.0;
                 }

                 for (int i = im1.ROI.Y; i < im1.ROI.Height; i++)
                 {
                     for (int j = im1.ROI.X; j < im1.ROI.Width; j++)
                     {
                         int val = im1.Data[i, j, 0];
                         hist[val]++;
                     }
                 }


                 for (int t = 0; t < 256; t++)
                     hist[t] /= (im1.Width * im1.Height);

                 double sumU = 0.0;
                 
                 for (int t = 0; t < 256; t++)
                 {
                     sumU += hist[t] * hist[t];
                 }


                 return sumU;

             }
             catch
             {
                 System.Console.WriteLine("Incapable de trouver l'Uniformite");
                 return 0;
             }
         }

		 public double EstimerCVAmax1Graine()
         {

             try
             {
				 int amax = typeG.AireMax;
				 double moySpheigne, moySeuilAmax1Graine, stdSeuilAmax1Graine;
				 moySpheigne = EstimerMoyenneSpheigne();
				 
				 Image<Gray, Byte> im1 = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();
				 
                 double[] hist = new double[256]; // histo de la spheigne
				 
                 for (int k = int (moySpheigne); k < 256; k++)
                 {
                     hist[k]  = 0.0;
                 }

                 for (int i = im1.ROI.Y+10; i < im1.ROI.Height-10; i++)
                 {
                     for (int j = im1.ROI.X+10; j < im1.ROI.Width-10; j++)
                     {
                         int val = im1.Data[i, j, 0];
						 if(val > int (moySpheigne))
							hist[val]++;
                     }
                 }

				int a=0; // aire sous la courbe de histo de 255 a t ou t correspond au nombre de pixel de graines AireMax
				int t=255; 
                 while (a < amax)
				 {
                     a+=hist[t--];
				 }
				// t++;
				 
				 // Calcul de la moyenne des amax pixels de la region de spheigne entre t et 255
				 moySeuilAmax1Graine = 0;
				 for (int j = t; j < 256; j++)
                 {
							moySeuilAmax1Graine += j*hist[j];
                 }
				 moySeuilAmax1Graine /= a;
				 
				 // Calcul de ecart type des amax pixels de la region de spheigne entre t et 255
				 stdSeuilAmax1Graine = 0;
				 for (int j = t; j < 256; j++)
                 {
							stdSeuilAmax1Graine += (j-moySeuilAmax1Graine)*(j-moySeuilAmax1Graine)*hist[j];
                 }
				 
				 stdSeuilAmax1Graine = Math.Sqrt(stdSeuilAmax1Graine /= a);

 
                 
                 double sigmuPourcent = (stdSeuilAmax1Graine/moySeuilAmax1Graine) * 100;


                 return sigmuPourcent;

             }
             catch
             {
                 System.Console.WriteLine("Incapable de trouver le coefficient de variation");
                 return 0;
             }
         }

		 public double EstimerSeuilAmax1Graine()
         {

             try
             {
				 int amax = typeG.AireMax;
				 double moySpheigne, moySeuilAmax1Graine, stdSeuilAmax1Graine;
				 moySpheigne = EstimerMoyenneSpheigne();
				 
				 Image<Gray, Byte> im1 = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();
				 
                 double[] hist = new double[256]; // histo de la spheigne
				 
                 for (int k = int (moySpheigne); k < 256; k++)
                 {
                     hist[k]  = 0.0;
                 }

                 for (int i = im1.ROI.Y+10; i < im1.ROI.Height-10; i++)
                 {
                     for (int j = im1.ROI.X+10; j < im1.ROI.Width-10; j++)
                     {
                         int val = im1.Data[i, j, 0];
						 if(val > int (moySpheigne))
							hist[val]++;
                     }
                 }

				int a=0; // aire sous la courbe de histo de 255 a t ou t correspond au nombre de pixel de graines AireMax
				int t=255; 
                 while (a < amax)
				 {
                     a+=hist[t--];
				 }
				// t++;
				 

                 return t;

             }
             catch
             {
                 System.Console.WriteLine("Incapable de trouver le seuil Amax");
                 return 0;
             }
         }
 /* ----------------------------------------------------------------------------------------------------------
          *  Fonction detectant la graine
          *  ---------------------------------------------------------------------------------------------------------*/
         public List<Graine> DetecterGraine(double moyenne) /*utilisation de la moyenne pour tresholder*/
         { 
            List<Graine> ListeGraine = new List<Graine>();
            
            Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();
           double std = DetecterStd(moyenne);
            //double moyenneSpheigne = DetecterMoyenneSpheigne(moyenne);
           //double stdSpheigne = DetecterStdSpheigne(moyenne);
            //double unif = DetecterUniformite(imageRGBgray, moyenne);
          
          //double seuil = TrouverSeuilOptimalCassette(imageRGBgray, moyenne);
           //imageRGBgray._ThresholdBinary(new Gray(seuil), new Gray(255)); // On rajoute 50

            //imageRGBgray._ThresholdBinary(new Gray(moyenneSpheigne +  2*stdSpheigne), new Gray(255));


          // if (std / moyenne <= 0.15) // cv faible donc cellule sans graines
          //     return ListeGraine;

           // imageRGBgray._ThresholdBinary(new Gray(moyenne + 50), new Gray(255)); // On rajoute 50
            imageRGBgray._ThresholdBinary(new Gray(moyenne + 2.0*std), new Gray(255)); // On rajoute 50

             // elemine les spikes
            imageRGBgray.Erode(1).Dilate(1);

            // emplir les trous
           //  imageRGBgray.Dilate(1).Erode(1);

             imageRGBgray.SmoothMedian(7);

             using (MemStorage storage = new MemStorage()) //allocate storage for contour approximation
             
             /*pour chaque contour dans une cellulle*/
             // on detecte les contour selon une grandeur de graine x
             for (Contour<Point> contours = imageRGBgray.FindContours(); contours != null; contours = contours.HNext)
             {
               //  if (contours.Total >= 6 && contours.Area > ((float)typeG.largeurMin * (float)typeG.hauteurMin * Math.PI) && contours.BoundingRectangle.Width > (float)typeG.largeurMin && contours.BoundingRectangle.Width < (float)typeG.largeurMax &&
                //          contours.BoundingRectangle.Height > (float)typeG.hauteurMin && contours.BoundingRectangle.Height < (float)typeG.hauteurMax)
                // if (contours.Total >= 6 && contours.Perimeter > typeG.pmin && contours.Area > typeG.aireMin && contours.Area < (3.0*typeG.aireMax))
                // {

            

                   if (contours.Total >= 6 && contours.Perimeter > 63 && contours.Area > 188)
                   {


                     bool ok = true;
                   /*  if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height -10 )
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height - 10 )
                         ok = false;

                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) < (imageRGBgray.Width / 2 - typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) > (imageRGBgray.Width / 2 + typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) < (imageRGBgray.Height / 2 - typeG.largeurMax / 2))
                         ok = false;

                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) >  (imageRGBgray.Height / 2 + typeG.largeurMax / 2))
                         ok = false;
                    * 
                    */ 
            
                    /* if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) < (imageRGBgray.Width / 2 - 20))
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) > (imageRGBgray.Width / 2 + 20))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) < (imageRGBgray.Height / 2 - 20))
                         ok = false;

                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) > (imageRGBgray.Height / 2 + 20))
                         ok = false;*/

                    // if ((contours.Area * 4.0 * Math.PI) / (contours.Perimeter * contours.Perimeter) > .95) // proche d un cercle ?????????
                   //      ok = false;

               //      if ((contours.GetMinAreaRect().size.Height / contours.GetMinAreaRect().size.Width) > 2.0)
                //         ok = false;






 
                     // si la graine se trouve a 10 pixel des coins du contours // pourquoi
                     if (ok )
                     {
                        
                        Graine gr = new Graine(contours, 1);
                        
                      //  if (gr.Localisation.X > 1  &&gr.Localisation.Y > 1 && imageRGBgray.Width - (gr.Localisation.X + contours.BoundingRectangle.Width) > 1
                       //       && imageRGBgray.Height - (gr.Localisation.Y + contours.BoundingRectangle.Height) > 1)
                      //  {

                        /*http://www.emgu.com/wiki/files/2.0.1.0/html/492603a1-20de-fa97-b128-bd8d55a29e48.htm*/
                /*         Seq<Emgu.CV.Structure.MCvConvexityDefect> te = contours.GetConvexityDefacts(storage, Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);
                            MCvConvexityDefect[] defacts = te.ToArray();
                            List<double> angle = new List<double>();
                            foreach (MCvConvexityDefect d in defacts)
                            {
                                double fA = Math.Sqrt(Math.Pow(d.EndPoint.X - d.DepthPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.DepthPoint.Y, 2)); //distance entre b et c
                                double fB = Math.Sqrt(Math.Pow(d.EndPoint.X - d.StartPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.StartPoint.Y, 2));  //distance entre a et c
                                double fC = Math.Sqrt(Math.Pow(d.DepthPoint.X - d.StartPoint.X, 2) + Math.Pow(d.DepthPoint.Y - d.StartPoint.Y, 2));
                                double fAngle = (Math.Pow(fA, 2) + Math.Pow(fC, 2) - Math.Pow(fB, 2)) / (2 * fA * fC); //(a2 + c2 - b2) / (2*a*c)
                                
                                if (Math.Abs(fAngle) < 0.6) // > 45 < 135 degres
                                    angle.Add(fAngle);
                          
                               // if (Math.Abs(fAngle) < (float)typeG.angle)
                                //    angle.Add(fAngle);

                                //if (Math.Acos(fAngle) < (float)typeG.angle)
                                 //   angle.Add(fAngle);
                               // if (Math.Acos(fAngle) < 135.0)
                                 //   angle.Add(fAngle);

                            }
                  
                 */
                            /*Calcule le nombre de graines à partir des angles */
                            //  System.Console.WriteLine(angle.Count);
                          //  gr.Nb_graine = angle.Count > 0 ? gr.Nb_graine + 1 : gr.Nb_graine;

                        /*    Seq<Point> cHull = contours.GetConvexHull(Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);

                            MCvBox2D minR = contours.GetMinAreaRect();

                            if (typeG.nom == "Pin blanc")
                            {
                                gr.Nb_graine = ((contours.Area  > 188) && (minR.size.Height > 43) && (minR.size.Width > 28)) ? gr.Nb_graine + 1 : gr.Nb_graine;
                            }*/


                           
                            //if((contours.Area/cHull.Area) < 0.9) // solidity 
                           // gr.Nb_graine = (((float)(contours.Area / cHull.Area) < typeG.solidite) && (contours.Area > typeG.aireMax)) ? gr.Nb_graine + 1 : gr.Nb_graine; // 1 graine ou plus que 2

                            //gr.Nb_graine = (((float)(contours.Area / cHull.Area) < 0.95) && !((contours.Area > typeG.aireMin) && (contours.Area < typeG.aireMax))) ? gr.Nb_graine + 1 : gr.Nb_graine;
                           // gr.Nb_graine = (((float)(contours.Area / cHull.Area) < 0.90) && (contours.Area > 2.0* typeG.aireMin)) ? gr.Nb_graine + 1 : gr.Nb_graine;
                         ListeGraine.Add(gr);
                        }
                   }
                // }
             
             }
             return ListeGraine;
         }


         public List<Graine> DetecterGraine3()
         {
             Image<Gray, byte> imageNirGray = imageNir.Convert<Gray, byte>().PyrDown().PyrUp();
             Image<Gray, byte> imageUvGray = imageUv.Convert<Gray, byte>().PyrDown().PyrUp();

             Gray averageNir;
             MCvScalar stdvNir;
             Gray averageUv;
             MCvScalar stdvUv;

             imageNirGray.AvgSdv(out averageNir, out stdvNir);
             imageNirGray = imageNirGray.ThresholdBinary(new Gray(averageNir.Intensity + 2*stdvNir.v0), new Gray(255));
             imageNirGray.Erode(1).Dilate(1);
             imageNirGray.Dilate(1).Erode(1);

             imageUvGray.AvgSdv(out averageUv, out stdvUv);
             imageUvGray = imageUvGray.ThresholdBinary(new Gray(averageUv.Intensity + 2*stdvUv.v0), new Gray(255));
             imageUvGray.Erode(1).Dilate(1);
             imageUvGray.Dilate(1).Erode(1);

             Image<Gray, byte> imageResultat = imageUvGray.AbsDiff(imageNirGray);


             List<Graine> ListeGraine = new List<Graine>();

             using (MemStorage storage = new MemStorage())

                 /*pour chaque contour dans une cellulle*/
                 // on detecte les contour selon une grandeur de graine x
                 for (Contour<Point> contours = imageResultat.FindContours(); contours != null; contours = contours.HNext)
                 {

                     bool ok = true;
                     if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageResultat.Width - 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageResultat.Height - 10)
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageResultat.Width - 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageResultat.Height - 10)
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) < (imageResultat.Width / 2 - typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) > (imageResultat.Width / 2 + typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) < (imageResultat.Height / 2 - typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) > (imageResultat.Height / 2 + typeG.largeurMax / 2))
                         ok = false;


                     // si la graine se trouve a 10 pixel des coins du contours // pourquoi
                     if (ok)
                     {

                         Graine gr = new Graine(contours, 1);


                         /*http://www.emgu.com/wiki/files/2.0.1.0/html/492603a1-20de-fa97-b128-bd8d55a29e48.htm*/
                         Seq<Emgu.CV.Structure.MCvConvexityDefect> te = contours.GetConvexityDefacts(storage, Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);
                         MCvConvexityDefect[] defacts = te.ToArray();
                         List<double> angle = new List<double>();
                         foreach (MCvConvexityDefect d in defacts)
                         {
                             double fA = Math.Sqrt(Math.Pow(d.EndPoint.X - d.DepthPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.DepthPoint.Y, 2)); //distance entre b et c
                             double fB = Math.Sqrt(Math.Pow(d.EndPoint.X - d.StartPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.StartPoint.Y, 2));  //distance entre a et c
                             double fC = Math.Sqrt(Math.Pow(d.DepthPoint.X - d.StartPoint.X, 2) + Math.Pow(d.DepthPoint.Y - d.StartPoint.Y, 2));
                             double fAngle = (Math.Pow(fA, 2) + Math.Pow(fC, 2) - Math.Pow(fB, 2)) / (2 * fA * fC); //(a2 + c2 - b2) / (2*a*c)

                             if (Math.Abs(fAngle) < 0.6) // > 45 < 135 degres
                                 angle.Add(fAngle);


                         }
                         /*Calcule le nombre de graines à partir des angles */
                         //  System.Console.WriteLine(angle.Count);
                         gr.Nb_graine = angle.Count > 0 ? gr.Nb_graine + 1 : gr.Nb_graine;

                         Seq<Point> cHull = contours.GetConvexHull(Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);

                         ListeGraine.Add(gr);
                     }

                 }
             return ListeGraine;

         }
		 /* ----------------------------------------------------------------------------------------------------------
          *  Fonction detectant la graine
          *  ---------------------------------------------------------------------------------------------------------*/
         public List<Graine> DetecterGraineMai2018(double moyenne) /*utilisation de la moyenne pour tresholder*/
         { 
            List<Graine> ListeGraine = new List<Graine>();
            
            Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();
   
		 
		  double cV = EstimerCVAmax1Graine();
		  if (cV > 30) // 30 % de dispersion 
			  return ListeGraine;
		  
            double moyenneSpheigne = EstimerMoyenneSpheigne();
            double stdSpheigne = EstimerStdSpheigne(moyenneSpheigne);
            //double unif = DetecterUniformite(imageRGBgray, moyenne);
          
          //double seuil = TrouverSeuilOptimalCassette(imageRGBgray, moyenne);
           //imageRGBgray._ThresholdBinary(new Gray(seuil), new Gray(255)); 
		   
		   // double seuil = EstimerSeuilAmax1Graine();
			//imageRGBgray._ThresholdBinary(new Gray(seuil), new Gray(255));
			
			//if(moyenneSpheigne +  2*stdSpheigne > 255)
			//	imageRGBgray._ThresholdBinary(new Gray(moyenneSpheigne +  1.5*stdSpheigne), new Gray(255));
			//else
            //imageRGBgray._ThresholdBinary(new Gray(moyenneSpheigne +  2*stdSpheigne), new Gray(255));


             // elemine les spikes
            imageRGBgray.Erode(1).Dilate(1);

            // emplir les trous
           //  imageRGBgray.Dilate(1).Erode(1);

             imageRGBgray.SmoothMedian(7);

             using (MemStorage storage = new MemStorage()) //allocate storage for contour approximation
             
             /*pour chaque contour dans une cellulle*/
             // on detecte les contour selon une grandeur de graine x
             for (Contour<Point> contours = imageRGBgray.FindContours(); contours != null; contours = contours.HNext)
             {
               //  if (contours.Total >= 6 && contours.Area > ((float)typeG.largeurMin * (float)typeG.hauteurMin * Math.PI) && contours.BoundingRectangle.Width > (float)typeG.largeurMin && contours.BoundingRectangle.Width < (float)typeG.largeurMax &&
                //          contours.BoundingRectangle.Height > (float)typeG.hauteurMin && contours.BoundingRectangle.Height < (float)typeG.hauteurMax)
                // if (contours.Total >= 6 && contours.Perimeter > typeG.pmin && contours.Area > typeG.aireMin && contours.Area < (3.0*typeG.aireMax))
                // {

            

                   if (contours.Total >= 6 && contours.Perimeter > 63 /*typeG.pmin*/ && contours.Area > 188 /*typeG.aireMin*/)
                   {


                     bool ok = true;
                   /*  if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height -10 )
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height - 10 )
                         ok = false;

                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) < (imageRGBgray.Width / 2 - typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) > (imageRGBgray.Width / 2 + typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) < (imageRGBgray.Height / 2 - typeG.largeurMax / 2))
                         ok = false;

                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) >  (imageRGBgray.Height / 2 + typeG.largeurMax / 2))
                         ok = false;
                    * 
                    */ 
            
                    /* if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) < (imageRGBgray.Width / 2 - 20))
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) > (imageRGBgray.Width / 2 + 20))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) < (imageRGBgray.Height / 2 - 20))
                         ok = false;

                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) > (imageRGBgray.Height / 2 + 20))
                         ok = false;*/

                    // if ((contours.Area * 4.0 * Math.PI) / (contours.Perimeter * contours.Perimeter) > .95) // proche d un cercle ?????????
                   //      ok = false;

               //      if ((contours.GetMinAreaRect().size.Height / contours.GetMinAreaRect().size.Width) > 2.0)
                //         ok = false;






 
                     // si la graine se trouve a 10 pixel des coins du contours // pourquoi
                     if (ok )
                     {
                        
                        Graine gr = new Graine(contours, 1);
                        
                      //  if (gr.Localisation.X > 1  &&gr.Localisation.Y > 1 && imageRGBgray.Width - (gr.Localisation.X + contours.BoundingRectangle.Width) > 1
                       //       && imageRGBgray.Height - (gr.Localisation.Y + contours.BoundingRectangle.Height) > 1)
                      //  {

                        /*http://www.emgu.com/wiki/files/2.0.1.0/html/492603a1-20de-fa97-b128-bd8d55a29e48.htm*/
                /*         Seq<Emgu.CV.Structure.MCvConvexityDefect> te = contours.GetConvexityDefacts(storage, Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);
                            MCvConvexityDefect[] defacts = te.ToArray();
                            List<double> angle = new List<double>();
                            foreach (MCvConvexityDefect d in defacts)
                            {
                                double fA = Math.Sqrt(Math.Pow(d.EndPoint.X - d.DepthPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.DepthPoint.Y, 2)); //distance entre b et c
                                double fB = Math.Sqrt(Math.Pow(d.EndPoint.X - d.StartPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.StartPoint.Y, 2));  //distance entre a et c
                                double fC = Math.Sqrt(Math.Pow(d.DepthPoint.X - d.StartPoint.X, 2) + Math.Pow(d.DepthPoint.Y - d.StartPoint.Y, 2));
                                double fAngle = (Math.Pow(fA, 2) + Math.Pow(fC, 2) - Math.Pow(fB, 2)) / (2 * fA * fC); //(a2 + c2 - b2) / (2*a*c)
                                
                                if (Math.Abs(fAngle) < 0.6) // > 45 < 135 degres
                                    angle.Add(fAngle);
                          
                               // if (Math.Abs(fAngle) < (float)typeG.angle)
                                //    angle.Add(fAngle);

                                //if (Math.Acos(fAngle) < (float)typeG.angle)
                                 //   angle.Add(fAngle);
                               // if (Math.Acos(fAngle) < 135.0)
                                 //   angle.Add(fAngle);

                            }
                  
                 */
                            /*Calcule le nombre de graines à partir des angles */
                            //  System.Console.WriteLine(angle.Count);
                          //  gr.Nb_graine = angle.Count > 0 ? gr.Nb_graine + 1 : gr.Nb_graine;

                        /*    Seq<Point> cHull = contours.GetConvexHull(Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);

                            MCvBox2D minR = contours.GetMinAreaRect();

                            if (typeG.nom == "Pin blanc")
                            {
                                gr.Nb_graine = ((contours.Area  > 188) && (minR.size.Height > 43) && (minR.size.Width > 28)) ? gr.Nb_graine + 1 : gr.Nb_graine;
                            }*/


                           
                            //if((contours.Area/cHull.Area) < 0.9) // solidity 
                           // gr.Nb_graine = (((float)(contours.Area / cHull.Area) < typeG.solidite) && (contours.Area > typeG.aireMax)) ? gr.Nb_graine + 1 : gr.Nb_graine; // 1 graine ou plus que 2

                            //gr.Nb_graine = (((float)(contours.Area / cHull.Area) < 0.95) && !((contours.Area > typeG.aireMin) && (contours.Area < typeG.aireMax))) ? gr.Nb_graine + 1 : gr.Nb_graine;
                           // gr.Nb_graine = (((float)(contours.Area / cHull.Area) < 0.90) && (contours.Area > 2.0* typeG.aireMin)) ? gr.Nb_graine + 1 : gr.Nb_graine;
                         ListeGraine.Add(gr);
                        }
                   }
                // }
             
             }
             return ListeGraine;
         }

    }
 }

